"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const child_process_1 = require("child_process");
const util_1 = require("util");
const framework_core_1 = require("@cloudbase/framework-core");
const lowcode_deployer_1 = __importStar(require("@cloudbase/lowcode-deployer"));
const SUPPORT_DEPLOY_MODE = ['upload', 'preview'];
const MP_CONFIG_FILENAME = 'project.config.json';
class MiniProgramsPlugin extends framework_core_1.Plugin {
    constructor(name, api, inputs) {
        super(name, api, inputs);
        this.name = name;
        this.api = api;
        this.inputs = inputs;
        const DEFAULT_INPUTS = {
            localPath: './',
            deployMode: 'preview',
            ignores: ['node_modules/**/*'],
        };
        this.resolvedInputs = resolveInputs(this.inputs, DEFAULT_INPUTS);
        this.buildOutput = {};
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('MiniProgramPlugin: init', Object.assign(Object.assign({}, this.resolvedInputs), { artifactOptions: '***' }));
            this._initCI();
        });
    }
    _initCI() {
        var _a;
        const { projectPath } = this.api;
        const { appid, privateKeyPath, privateKey, localPath, deployMode } = this.resolvedInputs;
        if (!appid) {
            throw new Error('小程序 appid 不能为空，请在 cloudbaserc.json 中指明 appid. 小程序 appid 一般可以在 project.config.json 中找到');
        }
        if (!privateKey) {
            if (!privateKeyPath || !fs_1.default.existsSync(path_1.default.resolve(projectPath, privateKeyPath))) {
                throw new Error('找不到小程序的部署私钥，请在 cloudbaserc.json 指明私钥文件路径 privateKeyPath. 小程序的部署私钥可在微信公众平台上登录后获取');
            }
        }
        else {
            const keyPath = path_1.default.resolve(projectPath, './private.key');
            const key = Buffer.from(privateKey, 'base64');
            fs_1.default.writeFileSync(keyPath, key.toString());
            this.resolvedInputs.privateKeyPath = keyPath;
        }
        if (deployMode && !SUPPORT_DEPLOY_MODE.includes(deployMode)) {
            throw new Error(`CloudBase Framework: 不支持的小程序部署模式 '${deployMode}'`);
        }
        if (!fs_1.default.existsSync(path_1.default.resolve(projectPath, localPath, MP_CONFIG_FILENAME))) {
            throw new Error('项目内找不到小程序配置文件 project.config.json，请在 cloudbaserc.json 中指明小程序应用的项目路径 localPath.');
        }
        this.deployer = new lowcode_deployer_1.default(Object.assign({ api: this.api, logger: this.api.logger, type: 'MP', mode: ((_a = this.resolvedInputs) === null || _a === void 0 ? void 0 : _a.deployMode) === 'upload' ? lowcode_deployer_1.EDeployMode.PRODUCTION : lowcode_deployer_1.EDeployMode.PREVIEW, projectPath: path_1.default.resolve(projectPath, localPath), ignores: this.resolvedInputs.ignores, mpAppId: appid, miniprogramOptions: {
                privateKeyPath: path_1.default.resolve(projectPath, this.resolvedInputs.privateKeyPath),
                uploadOptions: this.resolvedInputs.uploadOptions,
                previewOptions: this.resolvedInputs.previewOptions,
            }, artifactOptions: this.resolvedInputs.artifactOptions }, this.resolvedInputs.deployerOprions));
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    remove() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    genCode() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    build() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('MiniProgramPlugin: build', Object.assign(Object.assign({}, this.resolvedInputs), { artifactOptions: '***' }));
            const { build: buildCommand, install: installCommand } = ((_a = this.resolvedInputs) === null || _a === void 0 ? void 0 : _a.commands) || {};
            if (installCommand) {
                this.api.logger.info(installCommand);
                yield (0, util_1.promisify)(child_process_1.exec)(installCommand);
            }
            if (buildCommand) {
                this.api.logger.info(buildCommand);
                yield (0, util_1.promisify)(child_process_1.exec)(buildCommand);
            }
            yield this.deployer.genearteMiniprogramNPM();
            return;
        });
    }
    compile() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            switch ((_a = this.resolvedInputs) === null || _a === void 0 ? void 0 : _a.deployMode) {
                case 'upload':
                case 'preview': {
                    this.buildOutput = yield this.deployer.deploy();
                    break;
                }
                default: {
                    break;
                }
            }
            return {};
        });
    }
    deploy() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('MiniProgramPlugin: deploy', this.buildOutput);
            const { link, version } = this.buildOutput || {};
            if (link) {
                this.api.logger.info(`${this.api.emoji('🚀')} 小程序（预览版）部署成功，预览二维码地址：${this.api.genClickableLink(url_1.default.format({
                    protocol: 'file:',
                    host: link,
                }))}`);
            }
            else if (version) {
                this.api.logger.info(`${this.api.emoji('🚀')} 小程序（体验版v${version}）上传成功，请在小程序管理后台将其设置为体验版本`);
            }
            else {
                throw new Error(`invalid buildOutput: ${this.buildOutput}`);
            }
        });
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
exports.plugin = MiniProgramsPlugin;
